@@ -1,6 +1,8 @@
 test_that(
   "with missing token returns error", {
 
+    skip_on_cran()
+
     token <- Sys.getenv("FINBIF_ACCESS_TOKEN")
     Sys.unsetenv("FINBIF_ACCESS_TOKEN")
 
@@ -8,6 +10,11 @@ test_that(
 
     Sys.setenv(FINBIF_ACCESS_TOKEN = token)
 
+    expect_error(
+      fb_occurrence(restricted_api = "KEY"),
+      "Restricted API token declared but token is unset"
+    )
+
   }
 )
 
